@@ -23,6 +23,9 @@
 #ifndef ARRAY_H
 #define ARRAY_H
 
+#include <stdlib.h>
+#include <joedog/defs.h>
+
 /**
  * ARRAY object
  */
@@ -31,12 +34,17 @@ extern  size_t ARRAYSIZE;
 
 ARRAY  new_array();
 ARRAY  array_destroy(ARRAY this);
+ARRAY  array_destroyer(ARRAY this, method m);
+void   array_set_destroyer(ARRAY this, method m);
 void   array_push(ARRAY this, void *thing);
 void   array_npush(ARRAY this, void *thing, size_t len);
 void * array_get(ARRAY this, int index);
+void * array_remove (ARRAY this, int index);
+void * array_pop(ARRAY this);
 void * array_next(ARRAY this);
 void * array_prev(ARRAY this);
 size_t array_length(ARRAY this);
+char * array_to_string(ARRAY this);
 
 #endif/*ARRAY_H*/
 
