@@ -1,7 +1,7 @@
 /**
  * HTTP/HTTPS protocol support 
  *
- * Copyright (C) 2000-2014 by
+ * Copyright (C) 2000-2016 by
  * Jeffrey Fulmer - <jeff@joedog.org>, et al. 
  * This file is distributed as part of Siege 
  *
@@ -24,16 +24,29 @@
 #include <http.h>
 #include <stdio.h>
 #include <stdarg.h>
-#include <cookie.h>
+#ifdef  HAVE_ZLIB
+# include <zlib.h>
+#endif/*HAVE_ZLIB*/
+#include <cookies.h>
 #include <string.h>
 #include <util.h>
 #include <load.h>
+#include <page.h>
+#include <memory.h>
+#include <notify.h>
+#include <response.h>
 #include <joedog/defs.h>
 
-#define MAXFILE 10240
+#define MAXFILE 0x10000 // 65536
 
-private char *__parse_pair(char **str);
-private char *__dequote(char *str);
+#ifndef HAVE_ZLIB
+#define MAX_WBITS 1024 // doesn't matter - we don't use it
+#endif/*HAVE_ZLIB*/
+
+pthread_mutex_t __mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_cond_t  __cond  = PTHREAD_COND_INITIALIZER;
+
+private int     __gzip_inflate(int window, const char *src, int srcLen, const char *dst, int dstLen);
 
 /**
  * HTTPS tunnel; set up a secure tunnel with the
@@ -55,6 +68,7 @@ https_tunnel_request(CONN *C, char *host, int port)
     );    
     rlen = strlen(request); 
     echo ("%s", request);
+    C->encrypt = FALSE;
     if ((n = socket_write(C, request, rlen)) != rlen){
       NOTIFY(ERROR, "HTTP: unable to write to socket." );
       return FALSE;
@@ -106,18 +120,18 @@ http_get(CONN *C, URL U)
   char   encoding[512];
   char   *request;
   char   portstr[16];
-  char   fullpath[4096];
+  char   fullpath[MAX_COOKIE_SIZE*2];
   char   cookie[MAX_COOKIE_SIZE+8];
-  time_t now;
-  char * ifmod = url_get_if_modified_since(U);
-  char * ifnon = url_get_etag(U);
-
-  now = time(NULL);
+  char * ifnon = NULL;
+  char * ifmod = NULL; 
 
   memset(hoststr, '\0', sizeof hoststr);
   memset(cookie,  '\0', sizeof cookie);
   memset(portstr, '\0', sizeof portstr);
 
+  ifnon = cache_get_header(C->cache, C_ETAG, U);
+  ifmod = cache_get_header(C->cache, C_LAST, U);
+
   /* Request path based on proxy settings */
   if(auth_get_proxy_required(my.auth)){
     sprintf(
@@ -137,7 +151,7 @@ http_get(CONN *C, URL U)
    * Set the protocol and keepalive strings
    * based on configuration conditions....
    */
-  if (my.protocol == FALSE || my.get == TRUE) {
+  if (my.protocol == FALSE || my.get == TRUE || my.print == TRUE) {
     snprintf(protocol, sizeof(protocol), "HTTP/1.0");
   } else {
     snprintf(protocol, sizeof(protocol), "HTTP/1.1");
@@ -148,13 +162,15 @@ http_get(CONN *C, URL U)
     snprintf(keepalive, sizeof(keepalive), "close");
   }
 
-  get_cookie_header(pthread_self(), url_get_hostname(U), cookie); 
+  cookies_header(my.cookies, url_get_hostname(U), cookie);
   if (C->auth.www) {
     if (C->auth.type.www==DIGEST) {
       snprintf (
         authwww, sizeof(authwww), "%s", 
         auth_get_digest_header(my.auth, HTTP, C->auth.wchlg, C->auth.wcred, url_get_method_name(U), fullpath)
       );
+    } else if (C->auth.type.www==NTLM) {
+      snprintf(authwww, sizeof(authwww), "%s", auth_get_ntlm_header(my.auth, HTTP));
     } else {
       snprintf(authwww, sizeof(authwww), "%s", auth_get_basic_header(my.auth, HTTP));
     }
@@ -162,8 +178,8 @@ http_get(CONN *C, URL U)
   if (C->auth.proxy) {
     if (C->auth.type.proxy==DIGEST) {
       snprintf (
-        authwww, sizeof(authwww), "%s", 
-        auth_get_digest_header(my.auth, HTTP, C->auth.wchlg, C->auth.wcred, url_get_method_name(U), fullpath)
+        authpxy, sizeof(authpxy), "%s", 
+        auth_get_digest_header(my.auth, PROXY, C->auth.pchlg, C->auth.pcred, url_get_method_name(U), fullpath)
       );
     } else  {
       snprintf(authpxy, sizeof(authpxy), "%s", auth_get_basic_header(my.auth, PROXY));
@@ -199,7 +215,7 @@ http_get(CONN *C, URL U)
    request = (char*)xmalloc(mlen);
    memset(request, '\0', mlen);
    memset(encoding, '\0', sizeof(encoding));
-   if (! my.get) {
+   if (! my.get || ! my.print) {
      snprintf(encoding, sizeof(encoding), "Accept-Encoding: %s\015\012", my.encoding); 
    }
 
@@ -234,12 +250,12 @@ http_get(CONN *C, URL U)
    * XXX: I hate to use a printf here (as opposed to echo) but we
    * don't want to preface the headers with [debug] in debug mode
    */
-  if ((my.debug || my.get) && !my.quiet) { printf("%s\n", request); fflush(stdout); }
+  if ((my.debug || my.get || my.print) && !my.quiet) { printf("%s\n", request); fflush(stdout); }
   
   if (rlen == 0 || rlen > mlen) { 
     NOTIFY(FATAL, "HTTP %s: request buffer overrun!", url_get_method_name(U));
   }
-
+  //XXX: printf("%s", request);
   if ((socket_write(C, request, rlen)) < 0) {
     xfree(ifmod);
     xfree(ifnon);
@@ -267,13 +283,13 @@ http_post(CONN *C, URL U)
   char   protocol[16]; 
   char   keepalive[16];
   char   cookie[MAX_COOKIE_SIZE];
-  char   fullpath[4096];
+  char   fullpath[MAX_COOKIE_SIZE*2];
 
   memset(hoststr,  '\0', sizeof(hoststr));
-  memset(cookie,   '\0', sizeof(cookie));
-  memset(portstr,  '\0', sizeof portstr);
-  memset(protocol, '\0', sizeof portstr);
-  memset(keepalive,'\0', sizeof portstr);
+  memset(cookie,   '\0', MAX_COOKIE_SIZE);
+  memset(portstr,  '\0', sizeof(portstr));
+  memset(protocol, '\0', sizeof(protocol));
+  memset(keepalive,'\0', sizeof(keepalive));
 
   if (auth_get_proxy_required(my.auth)) {
    sprintf(
@@ -295,7 +311,7 @@ http_post(CONN *C, URL U)
    * Set the protocol and keepalive strings
    * based on configuration conditions.... 
    */
-  if (my.protocol == FALSE || my.get == TRUE) { 
+  if (my.protocol == FALSE || my.get == TRUE || my.print == TRUE) { 
     snprintf(protocol, sizeof(protocol), "HTTP/1.0");
   } else {
     snprintf(protocol, sizeof(protocol), "HTTP/1.1");
@@ -306,13 +322,15 @@ http_post(CONN *C, URL U)
     snprintf(keepalive, sizeof(keepalive), "close");
   }
 
-  get_cookie_header(pthread_self(), url_get_hostname(U), cookie);
+  cookies_header(my.cookies, url_get_hostname(U), cookie);
   if (C->auth.www) {
     if (C->auth.type.www==DIGEST) {
       snprintf (
         authwww, sizeof(authwww), "%s", 
         auth_get_digest_header(my.auth, HTTP, C->auth.wchlg, C->auth.wcred, url_get_method_name(U), fullpath)
       );
+    } else if(C->auth.type.www==NTLM) {
+      snprintf(authwww, sizeof(authwww), "%s", auth_get_ntlm_header(my.auth, HTTP));
     } else {
       snprintf(authwww, sizeof(authwww), "%s", auth_get_basic_header(my.auth, HTTP));
     }
@@ -320,8 +338,8 @@ http_post(CONN *C, URL U)
   if (C->auth.proxy) {
     if (C->auth.type.proxy==DIGEST) {
       snprintf (
-        authwww, sizeof(authwww), "%s", 
-        auth_get_digest_header(my.auth, HTTP, C->auth.wchlg, C->auth.wcred, url_get_method_name(U), fullpath)
+        authpxy, sizeof(authpxy), "%s", 
+        auth_get_digest_header(my.auth, HTTP, C->auth.pchlg, C->auth.pcred, url_get_method_name(U), fullpath)
       );
     } else  {
       snprintf(authpxy, sizeof(authpxy), "%s", auth_get_basic_header(my.auth, PROXY));
@@ -357,7 +375,7 @@ http_post(CONN *C, URL U)
    request = (char*)xmalloc(mlen);
    memset(request, '\0', mlen);
    memset(encoding, '\0', sizeof(encoding));
-   if (! my.get) {
+   if (! my.get || ! my.print) {
      snprintf(encoding, sizeof(encoding), "Accept-Encoding: %s\015\012", my.encoding); 
    }
 
@@ -384,7 +402,7 @@ http_post(CONN *C, URL U)
     (strlen(cookie) > 8)?cookie:"", 
     (strncasecmp(my.extra, "Accept:", 7)==0) ? "" : accept,
     encoding, my.uagent, my.extra, keepalive, url_get_conttype(U), (long)url_get_postlen(U)
-  ); 
+  );
 
   if (rlen < mlen) {
     memcpy(request + rlen, url_get_postdata(U), url_get_postlen(U));
@@ -392,7 +410,7 @@ http_post(CONN *C, URL U)
   }
   rlen += url_get_postlen(U);
  
-  if (my.get || my.debug) printf("%s\n\n", request);
+  if (my.get || my.debug || my.print) printf("%s\n\n", request);
 
   if (rlen == 0 || rlen > mlen) {
     NOTIFY(FATAL, "HTTP %s: request buffer overrun! Unable to continue...", url_get_method_name(U)); 
@@ -406,213 +424,126 @@ http_post(CONN *C, URL U)
   return TRUE;
 }
 
-void
-http_free_headers(HEADERS *h)
-{
-  xfree(h->redirect);
-  xfree(h->auth.realm.proxy);
-  xfree(h->auth.realm.www);
-  xfree(h);
-}
-
 /**
  * returns HEADERS struct
  * reads from http/https socket and parses
  * header information into the struct.
  */
-HEADERS *
+RESPONSE
 http_read_headers(CONN *C, URL U)
 { 
-  int  x;           /* while loop index      */
-  int  n;           /* assign socket_read    */
-  char c;           /* assign char read      */
-  HEADERS *h;       /* struct to hold it all */
-  char line[MAX_COOKIE_SIZE];  /* assign chars read     */
-  
-  h = xcalloc(sizeof(HEADERS), 1);
+  int  x;
+  int  n; 
+  char c; 
+  char line[MAX_COOKIE_SIZE]; 
+  RESPONSE resp = new_response();
   
   while (TRUE) {
     x = 0;
-    memset(&line, '\0', MAX_COOKIE_SIZE);
+    //memset(&line, '\0', MAX_COOKIE_SIZE); //VL issue #4
     while ((n = socket_read(C, &c, 1)) == 1) {
       if (x < MAX_COOKIE_SIZE - 1)
         line[x] = c; 
       else 
         line[x] = '\n';
-      echo("%c", c);  
-      if ((line[0] == '\n') || (line[1] == '\n')) { 
-        return h;
+      echo("%c", c);
+      if (x <= 1 && line[x] == '\n') { //VL issue #4, changed from (line[0] == '\n' || line[1] == '\n')
+			return resp; 
       }
       if (line[x] == '\n') break;
       x ++;
     }
-    line[x]=0;
-    /* strip trailing CR */
-    if (x > 0 && line[x-1] == '\r') line[x-1]=0;
+    line[x]='\0';
+
+    // string carriage return
+    if (x > 0 && line[x-1] == '\r') line[x-1]='\0';
+
     if (strncasecmp(line, "http", 4) == 0) {
-      strncpy( h->head, line, 8);
-      h->code = atoi(line + 9); 
+      response_set_code(resp, line);
     }
-    if (strncasecmp(line, "content-length: ", 16) == 0) { 
+    if (strncasecmp(line, CONTENT_TYPE, strlen(CONTENT_TYPE)) == 0) {
+      response_set_content_type(resp, line);
+    }
+    if (strncasecmp(line, CONTENT_ENCODING, strlen(CONTENT_ENCODING)) == 0) {
+      response_set_content_encoding(resp, line);
+    }
+    if (strncasecmp(line, CONTENT_LENGTH, strlen(CONTENT_LENGTH)) == 0) { 
+      response_set_content_length(resp, line);
       C->content.length = atoi(line + 16); 
     }
-    if (strncasecmp(line, "set-cookie: ", 12) == 0) {
+    if (strncasecmp(line, SET_COOKIE, strlen(SET_COOKIE)) == 0) {
       if (my.cookies) {
-        memset(h->cookie, '\0', sizeof(h->cookie));
-        strncpy(h->cookie, line+12, strlen(line));
-        add_cookie(pthread_self(), url_get_hostname(U), h->cookie);
+        char tmp[MAX_COOKIE_SIZE];
+        memset(tmp, '\0', MAX_COOKIE_SIZE);
+        strncpy(tmp, line+12, strlen(line));
+        cookies_add(my.cookies, tmp, url_get_hostname(U));
       }
     }
-    if (strncasecmp(line, "connection: ", 12 ) == 0) {
-      if (strncasecmp(line+12, "keep-alive", 10) == 0) {
-        h->keepalive = 1;
-      } else if (strncasecmp(line+12, "close", 5) == 0) {
-        h->keepalive = 0;
-      }
+    if (strncasecmp(line, CONNECTION, strlen(CONNECTION)) == 0) {
+      response_set_connection(resp, line);
     }
     if (strncasecmp(line, "keep-alive: ", 12) == 0) {
-      char *tmp    = "";
-      char *option = "", *value = "";
-      char *newline = (char*)line;
-      while ((tmp = __parse_pair(&newline)) != NULL) {
-        option = tmp;
-        while (*tmp && !ISSPACE((int)*tmp) && !ISSEPARATOR(*tmp))
-          tmp++;
-        *tmp++=0;
-        while (ISSPACE((int)*tmp) || ISSEPARATOR(*tmp))
-          tmp++;
-        value  = tmp;
-        while (*tmp)
-          tmp++;  
-        if (!strncasecmp(option, "timeout", 7)) {
-          if(value != NULL){
-            C->connection.timeout = atoi(value);
-          } else {
-            C->connection.timeout = 15;
-          }
-        }
-        if (!strncasecmp(option, "max", 3)) {
-          if(value != NULL){
-            C->connection.max = atoi(value);
-          } else {
-            C->connection.max = 0;
-          }
-        }
-      }
+      if (response_set_keepalive(resp, line) == TRUE) {
+        C->connection.timeout = response_get_keepalive_timeout(resp);
+        C->connection.max     = response_get_keepalive_max(resp);
+      } 
     }
-    if (strncasecmp(line, "location: ", 10) == 0) {
-      size_t len  = strlen(line);
-      h->redirect = xmalloc(len);
-      memcpy(h->redirect, line+10, len-10);
-      h->redirect[len-10] = 0;
+    if (strncasecmp(line, LOCATION, strlen(LOCATION)) == 0) {
+      response_set_location(resp, line);
     }
-    if (strncasecmp(line, "last-modified: ", 15) == 0) {
+    if (strncasecmp(line, LAST_MODIFIED, strlen(LAST_MODIFIED)) == 0) {
+      response_set_last_modified(resp, line);
       char *date;
       size_t len = strlen(line);
       if(my.cache){
         date = xmalloc(len);
         memcpy(date, line+15, len-14);
-        url_set_last_modified(U, date);
+        cache_add(C->cache, C_LAST, U, date);
         xfree(date); 
       }
     }
-    if (strncasecmp(line, "etag: ", 6) == 0) {
+    if (strncasecmp(line, ETAG, strlen(ETAG)) == 0) {
       char   *etag;
       size_t len = strlen(line);
-      if(my.cache){
-        etag = xmalloc(len);
+      if (my.cache) {
+        etag = (char *)xmalloc(len);
+        memset(etag, '\0', len);
         memcpy(etag, line+6, len-5);
-        etag[len-1] = '\0';
-        url_set_etag(U, etag);
+        cache_add(C->cache, C_ETAG, U, etag);
         xfree(etag);
       }
     }
-    if (strncasecmp(line, "www-authenticate: ", 18) == 0) {
-      char *tmp     = ""; 
-      char *option  = "", *value = "";
-      char *newline = (char*)line;
-      if (strncasecmp(line+18, "digest", 6) == 0) {
-        newline += 24;
-        h->auth.type.www      = DIGEST;
-        h->auth.challenge.www = xstrdup(line+18);
-      } else {
-        newline += 23;
-        h->auth.type.www = BASIC;
-      }
-      while ((tmp = __parse_pair(&newline)) != NULL) {
-        option = tmp; 
-        while (*tmp && !ISSPACE((int)*tmp) && !ISSEPARATOR(*tmp))
-          tmp++;
-        *tmp++=0;
-        while (ISSPACE((int)*tmp) || ISSEPARATOR(*tmp))
-          tmp++; 
-        value  = tmp;
-        while (*tmp)
-          tmp++;
-        if (!strncasecmp(option, "realm", 5)) {
-          if(value != NULL){
-	    h->auth.realm.www = xstrdup(__dequote(value));
-            url_set_realm(U, __dequote(value));
-          } else {
-            h->auth.realm.www = xstrdup("");
-          }
-        }
-      } /* end of parse pairs */
+    if (strncasecmp(line, WWW_AUTHENTICATE, strlen(WWW_AUTHENTICATE)) == 0) {
+      response_set_www_authenticate(resp, line);
     } 
-    if (strncasecmp(line, "proxy-authenticate: ", 20) == 0) {
-      char *tmp     = ""; 
-      char *option  = "", *value = "";
-      char *newline = (char*)line;
-      if(strncasecmp(line+20, "digest", 6) == 0){
-        newline += 26;
-        h->auth.type.proxy      = DIGEST;
-        h->auth.challenge.proxy = xstrdup(line+20);
-      } else {
-        newline += 25;
-        h->auth.type.proxy = BASIC;
-      }
-      while((tmp = __parse_pair(&newline)) != NULL){
-        option = tmp; 
-        while(*tmp && !ISSPACE((int)*tmp) && !ISSEPARATOR(*tmp))
-          tmp++;
-        *tmp++=0;
-        while(ISSPACE((int)*tmp) || ISSEPARATOR(*tmp))
-          tmp++; 
-        value  = tmp;
-        while(*tmp)
-          tmp++;
-        if (!strncasecmp(option, "realm", 5)) {
-          if (value != NULL) {
-	    h->auth.realm.proxy = xstrdup(__dequote(value));
-          } else {
-            h->auth.realm.proxy = xstrdup("");
-          }
-        }
-      } /* end of parse pairs */
+    if (strncasecmp(line, PROXY_AUTHENTICATE, strlen(PROXY_AUTHENTICATE)) == 0) {
+      response_set_proxy_authenticate(resp, line);
+    }                     
+    if (strncasecmp(line, TRANSFER_ENCODING, strlen(TRANSFER_ENCODING)) == 0) {
+      response_set_transfer_encoding(resp, line);
     }
-    if (strncasecmp(line, "transfer-encoding: ", 19) == 0) {
-      if (strncasecmp(line+20, "chunked", 7)) {
-        C->content.transfer = CHUNKED; 
-      } else if (strncasecmp(line+20, "trailer", 7)) {
-        C->content.transfer = TRAILER; 
-      } else {
-        C->content.transfer = NONE;
+    if (strncasecmp(line, EXPIRES, strlen(EXPIRES)) == 0) {
+      char   *expires;
+      size_t  len = strlen(line); 
+      if (my.cache) {
+        expires = (char *)xmalloc(len);
+        memset(expires, '\0', len);
+        memcpy(expires, line+9, len-8);
+        cache_add(C->cache, C_EXPIRES, U, expires);
+        xfree(expires);
       }
     }
-    if (strncasecmp(line, "expires: ", 9) == 0) {
-      /* printf("%s\n", line+10);  */
-    }
     if (strncasecmp(line, "cache-control: ", 15) == 0) {
       /* printf("%s\n", line+15); */
     }
     if (n <=  0) { 
       echo ("read error: %s:%d", __FILE__, __LINE__);
-      http_free_headers(h);
-      return(NULL); 
+      resp = response_destroy(resp);
+      return resp; 
     } /* socket closed */
   } /* end of while TRUE */
 
-  return h;
+  return resp;
 }
 
 int
@@ -631,7 +562,7 @@ http_chunk_size(CONN *C)
   if (((C->chkbuf[0] == '\n')||(strlen(C->chkbuf)==0)||(C->chkbuf[0] == '\r'))) {
     return -1;
   }
- 
+
   errno  = 0;
   if (!isxdigit((unsigned)*C->chkbuf))
     return -1;
@@ -645,166 +576,150 @@ http_chunk_size(CONN *C)
   return -1;
 }
   
-/**
- * returns ssize_t
- */
 ssize_t
-http_read(CONN *C)
+http_read(CONN *C, RESPONSE resp)
 { 
   int    n      = 0;
-  char   c;
   int    chunk  = 0;
   size_t bytes  = 0;
   size_t length = 0;
-  static char body[MAXFILE];
+  char   dest[MAXFILE*6];
+  char   *ptr = NULL;
+  char   *tmp = NULL;
+  size_t size = MAXFILE; 
 
-  if (C == NULL) NOTIFY(FATAL, "Connection is NULL! Unable to proceed"); 
-
-  if (my.get || my.debug) {
-    // The is debug / get mode - read a char and print it to stdout
-    if (C->content.length > 0) {
-      length = (C->content.length < MAXFILE) ? C->content.length:MAXFILE;
-      do {
-        if ((n = socket_read(C, &c, 1)) == 0)
-          break;
-        echo ("%c", c);
-        bytes += n;
-        length = (C->content.length - bytes < MAXFILE)?C->content.length-bytes:MAXFILE;
-      } while (bytes < C->content.length);
-    } else {
-      do {
-        /*if (my.chunked && C->content.transfer == CHUNKED) {
-          chunk = http_chunk_size(C);
-        } */
-        if ((n = socket_read(C, &c, 1)) == 0)
-          break;
-        echo ("%c", c);
-        bytes += n;
-      } while(TRUE);
-    }
-  } else {
-    // This is non-debug / non-get mode. We'll read into a larger bucket
-    if (C->content.length > 0) {
-      length = (C->content.length < MAXFILE) ? C->content.length:MAXFILE;
-      do {
-        memset(body, '\0', sizeof(body));
-        if ((n = socket_read(C, body, length)) == 0)
-          break;
-        bytes += n;
-        length = (C->content.length - bytes < MAXFILE)?C->content.length-bytes:MAXFILE; 
-      } while (bytes < C->content.length); 
-    } else if (my.chunked && C->content.transfer == CHUNKED) {
-      int tries = 0;
-      while (tries < 128) {
-        chunk = http_chunk_size(C);
-        if (chunk == 0)
-          break;
-        else if (chunk < 0) {
-          tries ++;
-          continue;
-        }
-        do {
-          int n;
-          memset(body, '\0', MAXFILE);
-          n = socket_read(C, body, (chunk>MAXFILE)?MAXFILE:chunk);
-          chunk -= n;
-          bytes += n;
-        } while(chunk > 0);
-      }
-    } else {
-      do {
-        memset(body, '\0', sizeof(body));
-        if ((n = socket_read(C, body, sizeof(body))) == 0)
-          break;
-        bytes += n;
-      } while(TRUE);
-    }
+  if (C == NULL) {
+	  NOTIFY(FATAL, "Connection is NULL! Unable to proceed"); 
+	  return 0;
   }
-      
-  //if (my.debug||my.get) { printf("\n"); fflush(stdout); }
-  echo ("\n"); 
+
+  if (C->content.length == 0) //VL
+	  return 0;
+  else if (C->content.length == (size_t)~0L)
+	  C->content.length = 0; //not to break code below...
   
+  memset(dest, '\0', sizeof dest);
+
+  //pthread_mutex_lock(&__mutex);  //VL - moved
+  
+  if (C->content.length > 0) {
+    length = C->content.length;
+    ptr    = xmalloc(length+1);
+    memset(ptr, '\0', length+1);
+    do {
+      if ((n = socket_read(C, ptr, length)) == 0) {
+        break;
+      }
+      bytes += n;
+    } while (bytes < length); 
+  } else if (my.chunked && response_get_transfer_encoding(resp) == CHUNKED) {
+    int  r = 0;
+    bytes  = 0;
+    BOOLEAN done = FALSE;
+
+    ptr = xmalloc(size);
+    memset(ptr, '\0', size);
+
+    do {
+      chunk = http_chunk_size(C);
+      if (chunk == 0){
+        socket_readline(C, C->chkbuf, sizeof(C->chkbuf)); //VL - issue #3
+        break;
+      } else if (chunk < 0) {
+        chunk = 0;
+        continue;
+      }  
+      while (n < chunk) {
+        int remaining_in_chunk = chunk - n;
+        int space_in_buf = size - bytes;
+        int to_read = remaining_in_chunk < space_in_buf ? remaining_in_chunk : space_in_buf;
+        r = socket_read(C, &ptr[bytes], to_read); 
+        bytes += r;
+        if (r <= 0) {
+          done = TRUE;
+          break;
+        }
+        n += r;
+        if (bytes >= size) {
+          tmp = realloc(ptr, size*2);
+          if (tmp == NULL) {
+            free(ptr);
+            return -1; 
+          }
+          ptr   = tmp;
+          size *= 2;
+        }
+      }
+      n = 0;
+    } while (! done);
+    ptr[bytes] = '\0';
+  } else {
+    ptr = xmalloc(size);
+    memset(ptr, '\0', size);
+    do {
+      n = socket_read(C, ptr, size);
+      bytes += n;
+      if (n <= 0) { 
+        break;
+      }
+    } while (TRUE);
+  }
+
+  if (response_get_content_encoding(resp) == GZIP) {
+    __gzip_inflate(MAX_WBITS+32, ptr, bytes, dest, sizeof(dest));
+  }
+  if (response_get_content_encoding(resp) == DEFLATE) {
+    __gzip_inflate(-MAX_WBITS, ptr, bytes, dest, sizeof(dest));
+  }
+  if (strlen(dest) > 0) {
+    page_concat(C->page, dest, strlen(dest));
+  } else {
+    page_concat(C->page, ptr, strlen(ptr));
+  }
+  xfree(ptr);
+  echo ("\n");
+  //pthread_mutex_unlock(&__mutex);
   return bytes;
 }
 
-
-/**
- * parses option=value pairs from an
- * http header, see keep-alive: above
- * while(( tmp = __parse_pair( &newline )) != NULL ){
- *   do_something( tmp );
- * }
- */
-private char *
-__parse_pair(char **str)
+private int
+__gzip_inflate(int window, const char *src, int srcLen, const char *dst, int dstLen)
 {
-  int  okay  = 0;
-  char *p    = *str;
-  char *pair = NULL;
- 
-  if( !str || !*str ) return NULL;
-  /**
-   * strip the header label
-   */
-  while( *p && *p != ' ' )
-    p++;
-  *p++=0;
-  if( !*p ){
-    *str   = p;
-    return NULL;
-  }
- 
-  pair = p;
-  while( *p && *p != ';' && *p != ',' ){
-    if( !*p ){
-      *str = p;
-      return NULL;
+  int err=-1;
+
+#ifndef HAVE_ZLIB
+  NOTIFY(ERROR,
+    "gzip transfer-encoding requires zlib (%d, %d, %d)", window, srcLen, dstLen
+  );
+  dst = strdup(src);
+  (void)(dst); // shut the compiler up....
+  return err;
+#else
+  z_stream strm;
+  int ret        = -1;
+  strm.zalloc    = Z_NULL;
+  strm.zfree     = Z_NULL;
+  strm.opaque    = Z_NULL;
+  strm.avail_in  = srcLen;
+  strm.avail_out = dstLen;
+  strm.next_in   = (Bytef *)src;
+  strm.next_out  = (Bytef *)dst;
+
+  err = inflateInit2(&strm, window);
+  if (err == Z_OK) {
+    err = inflate(&strm, Z_FINISH);
+    if (err == Z_STREAM_END){
+      ret = strm.total_out;
+    } else {
+      inflateEnd(&strm);
+      return err;
     }
-    if( *p == '=' ) okay = 1;
-    p++;
+  } else {
+    inflateEnd(&strm);
+    return err;
   }
-  *p++ = 0;
-  *str = p;
- 
-  if( okay )
-    return pair;
-  else
-    return NULL;
-} 
-
-char *
-__rquote(char *str)
-{
-  char *ptr;
-  int   len;
-
-  len = strlen(str);
-  for(ptr = str + len - 1; ptr >= str && ISQUOTE((int)*ptr ); --ptr);
-
-  ptr[1] = '\0';
-
-  return str;
+  inflateEnd(&strm);
+  return ret;
+#endif/*HAVE_ZLIB*/
 }
 
-char *
-__lquote(char *str)
-{
-  char *ptr;
-  int  len;
-
-  for(ptr = str; *ptr && ISQUOTE((int)*ptr); ++ptr);
-
-  len = strlen(ptr);
-  memmove(str, ptr, len + 1);
-
-  return str;
-}
-
-char *
-__dequote(char *str)
-{
-  char *ptr;
-  ptr = __rquote(str);
-  str = __lquote(ptr);
-  return str;
-}
