@@ -1,7 +1,7 @@
 /**
  * Siege environment initialization.
  *
- * Copyright (C) 2000-2014 by
+ * Copyright (C) 2000-2015 by
  * Jeffrey Fulmer - <jeff@joedog.org>, et al. 
  * This file is distributed as part of Siege 
  *
@@ -25,31 +25,71 @@
 #include <util.h>
 #include <hash.h>
 #include <eval.h>
+#include <perl.h>
+#include <memory.h>
+#include <notify.h>
 #include <fcntl.h>
+#include <stralloc.h>
 #include <version.h>
 #include <joedog/boolean.h>
+#include <joedog/path.h>
 #include <joedog/defs.h>
-#include <joedog/joedog.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
+#include <stdlib.h>
+
+#define LINESZ 1024
 
 int
 init_config( void )
 {
   char *e;
+  char dir[256];
   int  res;
   struct stat buf;
+  BOOLEAN needed = FALSE;
 
-  /* Check if we were passed the -R switch to use a different siegerc file.
+  /**
+   * Beginning with siege-3.1.4 the local configuration was 
+   * moved to $HOME/.siege/siege.conf Let's configure ourselves
+   */
+  memset(dir, '\0', sizeof(dir));
+  snprintf(dir, sizeof(dir), "%s/.siege", getenv("HOME"));
+  if (stat(dir, &buf) < 0 && ENOENT == errno) {
+    needed = TRUE;
+  } else {
+    if (! S_ISDIR(buf.st_mode)) {
+      if (unlink(dir) == 0) needed = TRUE;
+    }
+  } 
+
+  /**
+   * We need to create a $HOME/.siege dir and configur ourselves.
+   * NOTE: We're going to assume siege.config is installed and in
+   * the PATH. If this is not the case, then blame the distributor
+   */  
+  while (needed) {
+    int ret; 
+
+    mkdir(dir, 0750);
+    ret = system("siege.config");
+    if (WIFSIGNALED(ret) && (WTERMSIG(ret) == SIGINT || WTERMSIG(ret) == SIGQUIT))
+      break;
+    needed = FALSE;
+  }
+
+  /** 
+   * Check if we were passed the -R switch to use a different siegerc file.
    * If not, check for the presence of the SIEGERC variable, otherwise
-   * use default of ~/.siegerc */
+   * use default of ~/.siege/siege.conf 
+   */
   if(strcmp(my.rc, "") == 0){
     if((e = getenv("SIEGERC")) != NULL){
       snprintf(my.rc, sizeof(my.rc), "%s", e);
     } else {
-      snprintf(my.rc, sizeof(my.rc), "%s/.siegerc", getenv("HOME"));
+      snprintf(my.rc, sizeof(my.rc), "%s/.siege/siege.conf", getenv("HOME"));
       if (stat(my.rc, &buf) < 0 && errno == ENOENT) {
         snprintf(my.rc, sizeof(my.rc), CNF_FILE);
       }
@@ -58,13 +98,15 @@ init_config( void )
 
   my.debug          = FALSE;
   my.quiet          = FALSE;
+  my.color          = TRUE;
   my.internet       = FALSE;
   my.config         = FALSE;
-  my.cookies        = TRUE;
   my.csv            = FALSE;
   my.fullurl        = FALSE;
   my.escape         = TRUE;
+  my.parser         = FALSE;
   my.secs           = -1;
+  my.limit          = 255;
   my.reps           = MAXREPS; 
   my.bids           = 5;
   my.login          = FALSE;
@@ -86,17 +128,22 @@ init_config( void )
   my.ssl_key        = NULL;
   my.ssl_ciphers    = NULL; 
   my.lurl           = new_array();
+  my.cookies        = new_cookies();
+  my.nomap          = xcalloc(1, sizeof(LINES));
+  my.nomap->index   = 0;
+  my.nomap->line    = NULL;
 
-  if((res = pthread_mutex_init(&(my.lock), NULL)) != 0)
+  if ((res = pthread_mutex_init(&(my.lock), NULL)) != 0)
     NOTIFY(FATAL, "unable to initiate lock");
-  if((res = pthread_cond_init(&(my.cond), NULL )) != 0)
+  if ((res = pthread_cond_init(&(my.cond), NULL )) != 0)
     NOTIFY(FATAL, "unable to initiate condition");
 
   if (load_conf(my.rc) < 0) {
-    fprintf( stderr, "****************************************************\n" );
+    /** Beginning with siege-3.1.4 we should never get here (in theory) **/
+    fprintf( stderr, "**************************************************\n" );
     fprintf( stderr, "siege: could not open %s\n", my.rc );
-    fprintf( stderr, "run \'siege.config\' to generate a new .siegerc file\n" );
-    fprintf( stderr, "****************************************************\n" );
+    fprintf( stderr, "run \'siege.config\' to generate a new config file\n" );
+    fprintf( stderr, "**************************************************\n" );
     return -1;
   }
   
@@ -126,7 +173,7 @@ init_config( void )
 
   if (strlen(my.logfile) < 1) 
     snprintf( 
-      my.logfile, sizeof( my.logfile ),
+      my.logfile, sizeof(my.logfile),
       "%s", LOG_FILE 
     );
   return 0;  
@@ -145,107 +192,142 @@ show_config(int EXIT)
       method = strdup("HEAD");
       break;
   }
-  
-  printf( "CURRENT  SIEGE  CONFIGURATION\n" );
-  printf( "%s\n", my.uagent ); 
-  printf( "Edit the resource file to change the settings.\n" );
-  printf( "----------------------------------------------\n" );
-  printf( "version:                        %s\n", version_string );
-  printf( "verbose:                        %s\n", my.verbose?"true":"false" );
-  printf( "quiet:                          %s\n", my.quiet?"true":"false" );
-  printf( "debug:                          %s\n", my.debug?"true":"false" );
-  printf( "protocol:                       %s\n", my.protocol?"HTTP/1.1":"HTTP/1.0" );
-  printf( "get method:                     %s\n", method);
+  printf("CURRENT  SIEGE  CONFIGURATION\n");
+  printf("%s\n", my.uagent); 
+  printf("Edit the resource file to change the settings.\n");
+  printf("----------------------------------------------\n");
+  printf("version:                        %s\n", version_string);
+  printf("verbose:                        %s\n", my.verbose  ? "true"     : "false");
+  printf("color:                          %s\n", my.color  ? "true"     : "false");
+  printf("quiet:                          %s\n", my.quiet    ? "true"     : "false");
+  printf("debug:                          %s\n", my.debug    ? "true"     : "false");
+  printf("protocol:                       %s\n", my.protocol ? "HTTP/1.1" : "HTTP/1.0");
+  printf("HTML parser:                    %s\n", my.parser   ? "enabled"  : "disabled");
+  printf("get method:                     %s\n", method);
   if (auth_get_proxy_required(my.auth)){
     printf("proxy-host:                     %s\n", auth_get_proxy_host(my.auth));
     printf("proxy-port:                     %d\n", auth_get_proxy_port(my.auth));
   }
-  printf( "connection:                     %s\n", my.keepalive?"keep-alive":"close" );
-  printf( "concurrent users:               %d\n", my.cusers );
-  if( my.secs > 0 )
-    printf( "time to run:                    %d seconds\n", my.secs );
+  printf("connection:                     %s\n", my.keepalive?"keep-alive":"close");
+  printf("concurrent users:               %d\n", my.cusers);
+  if (my.secs > 0)
+    printf( "time to run:                    %d seconds\n", my.secs);
   else
-    printf( "time to run:                    n/a\n" );
-  if(( my.reps > 0 )&&( my.reps != MAXREPS ))
-    printf( "repetitions:                    %d\n", my.reps );
+    printf( "time to run:                    n/a\n");
+  if ((my.reps > 0) && (my.reps != MAXREPS))
+    printf( "repetitions:                    %d\n", my.reps);
   else
-    printf( "repetitions:                    n/a\n" );
-  printf( "socket timeout:                 %d\n", my.timeout );
-  printf( "accept-encoding:                %s\n", my.encoding);
-  printf( "delay:                          %d sec%s\n", my.delay,my.delay>1?"s":"" );
-  printf( "internet simulation:            %s\n", my.internet?"true":"false"  );
-  printf( "benchmark mode:                 %s\n", my.bench?"true":"false"  );
-  printf( "failures until abort:           %d\n", my.failures );
-  printf( "named URL:                      %s\n", my.url==NULL||strlen(my.url)<2?"none":my.url );
-  printf( "URLs file:                      %s\n", strlen(my.file)>1?my.file:URL_FILE );
-  printf( "logging:                        %s\n", my.logging?"true":"false" );
-  printf( "log file:                       %s\n", my.logfile==NULL?LOG_FILE:my.logfile );
-  printf( "resource file:                  %s\n", my.rc);
-  printf( "timestamped output:             %s\n", my.timestamp?"true":"false");
-  printf( "comma separated output:         %s\n", my.csv?"true":"false");
-  printf( "allow redirects:                %s\n", my.follow?"true":"false" );
-  printf( "allow zero byte data:           %s\n", my.zero_ok?"true":"false" ); 
-  printf( "allow chunked encoding:         %s\n", my.chunked?"true":"false" ); 
-  printf( "upload unique files:            %s\n", my.unique?"true":"false" ); 
-  //printf( "proxy auth:                     " ); display_authorization( PROXY );printf( "\n" );
-  //printf( "www auth:                       " ); display_authorization( WWW ); 
-  printf( "\n" );
+    printf("repetitions:                    n/a\n" );
+  printf("socket timeout:                 %d\n", my.timeout);
+  printf("cache enabled:                  %s\n", my.cache==TRUE ? "true" : "false");
+  printf("accept-encoding:                %s\n", my.encoding);
+  printf("delay:                          %.3f sec%s\n", my.delay, (my.delay > 1) ? "s" : "");
+  printf("internet simulation:            %s\n", my.internet?"true":"false");
+  printf("benchmark mode:                 %s\n", my.bench?"true":"false");
+  printf("failures until abort:           %d\n", my.failures);
+  printf("named URL:                      %s\n", my.url==NULL||strlen(my.url) < 2 ? "none" : my.url);
+  printf("URLs file:                      %s\n", strlen(my.file) > 1 ? my.file : URL_FILE);
+  printf("thread limit:                   %d\n", (my.limit < 1) ? 255 : my.limit);
+  printf("logging:                        %s\n", my.logging ? "true" : "false");
+  printf("log file:                       %s\n", (my.logfile == NULL) ? LOG_FILE : my.logfile);
+  printf("resource file:                  %s\n", my.rc);
+  printf("timestamped output:             %s\n", my.timestamp?"true":"false");
+  printf("comma separated output:         %s\n", my.csv?"true":"false");
+  printf("allow redirects:                %s\n", my.follow?"true":"false");
+  printf("allow zero byte data:           %s\n", my.zero_ok?"true":"false"); 
+  printf("allow chunked encoding:         %s\n", my.chunked?"true":"false"); 
+  printf("upload unique files:            %s\n", my.unique?"true":"false"); 
+  if (my.parser == TRUE && my.nomap->index > 0) {
+    int i;
+    printf("no-follow:\n"); 
+    for (i = 0; i < my.nomap->index; i++) {
+      printf(" - %s\n", my.nomap->line[i]);
+    } 
+  }
+  //printf("proxy auth:                     " ); display_authorization(PROXY);printf("\n");
+  //printf("www auth:                       " ); display_authorization(WWW); 
+  printf("\n");
+
   xfree(method);
+  my.auth    = auth_destroy(my.auth);
+  my.lurl    = array_destroy(my.lurl);
+  my.cookies = cookies_destroy(my.cookies); 
+
   if (EXIT) exit(0);
   else return 0;
 }
 
-static char *
-get_line(FILE *fp)
+int
+readline(char **s, FILE *fp)
 {
+  int  c;
+  int  i;
+  int  len  = 0;
+  int  size = 0;
+  char *tmp = NULL;
   char *ptr = NULL;
-  char *newline;
-  char tmp[256];
 
-  memset( tmp, 0, sizeof(tmp)); 
-  do {
-    if((fgets(tmp, sizeof(tmp), fp)) == NULL) return(NULL);
-    if(ptr == NULL) {
-      ptr = xstrdup( tmp );
-    } else {
-      ptr = (char*)xrealloc(ptr, strlen(ptr) + strlen(tmp) + 1);
-      strcat( ptr, tmp );
+  ptr  = xmalloc(LINESZ);
+  size = LINESZ;
+
+  while ((c = fgetc(fp))!=EOF && c!='\n') {
+    if (len >= size) {
+      // the buffer is full - extend it
+      tmp = realloc(ptr, size + LINESZ);
+      if (tmp == NULL) {
+        free(ptr);
+        return -1;
+      }
+      ptr = tmp;
+      size += LINESZ;
     }
-    newline = strchr(ptr, '\n');
-  } while( newline == NULL );
-  *newline = '\0';
- 
-  return ptr;
-} 
-
-static char *
-chomp_line(FILE *fp, char **mystr, int *line_num)
-{
-  char *ptr;
-  while(TRUE){
-    if((*mystr = get_line( fp )) == NULL) return NULL;
-    (*line_num)++;
-    ptr = chomp(*mystr);
-    /* exclude comments */
-    if(*ptr != '#' && *ptr != '\0'){
-      return(ptr);
-    } else {
-      xfree(ptr);
+    ptr[len++] = (char)c;
+  }
+  if (len == 0) {
+    if (c == EOF) { /* empty file or read error */
+      free(ptr);
+      return -1;
+    } else { /* empty line */
+      ptr[0] = '\0';
+      *s = ptr;
+      return 0;
     }
   }
-} 
+
+  if (len+1 != size) { /* no room for '\0' */
+    tmp = realloc(ptr, len+1);
+    if (tmp == NULL) {
+      free(ptr);
+      return -1;
+    }
+    ptr = tmp;
+  }
+  ptr[len] = '\0';
+
+  for (i = 0; ptr[i] != '\0'; i++) {
+    if (ptr[i] == '#') {
+      ptr[i] = '\0';
+    }
+  }
+  *s = ptr;
+
+  return len;
+}
 
 int
 load_conf(char *filename)
 {
-  FILE *fp;
-  HASH H;
-  int  line_num = 0;
-  char *line;
-  char *tmp;
-  char *option;
-  char *optionptr;
-  char *value;
+  FILE    *fp;
+  HASH    H;
+  char    *line;
+  char    *option;
+  char    *optionptr;
+  char    *value;
+#ifdef  HAVE_ZLIB
+  BOOLEAN zlib = TRUE;
+#else  
+  BOOLEAN zlib = FALSE;
+#endif/*HAVE_ZLIB*/
  
   if ((fp = fopen(filename, "r")) == NULL) {
     return -1;
@@ -253,13 +335,20 @@ load_conf(char *filename)
 
   H = new_hash();
 
-  while ((line = chomp_line(fp, &line, &line_num)) != NULL) {
-    tmp = trim(line);
+  while (readline(&line, fp) != -1) {
+    char *tmp = line;
+    line = trim(line);
+    if (*line == '#' || *line == '\0' || strlen(line) < 1) {
+      free(line);
+      continue;
+    }
+
     optionptr = option = xstrdup(line);
     while (*optionptr && !ISSPACE((int)*optionptr) && !ISSEPARATOR(*optionptr)) {
       optionptr++;
     }
-    *optionptr++=0;
+    *optionptr++='\0';
+
     while (ISSPACE((int)*optionptr) || ISSEPARATOR(*optionptr)) {
       optionptr++;
     }
@@ -278,12 +367,31 @@ load_conf(char *filename)
       else
         my.verbose = FALSE;
     } 
+    else if (strmatch(option, "color")) {
+      if (strmatch(value, "false") || strmatch(value, "off"))
+        my.color = FALSE;
+      else
+        my.color = TRUE;
+    } 
     else if (strmatch(option, "quiet")) {
       if (!strncasecmp(value, "true", 4))
         my.quiet = TRUE;
       else
         my.quiet = FALSE;
     } 
+    else if (strmatch(option, "parser")) {
+      if (!strncasecmp(value, "true", 4))
+        my.parser = TRUE;
+      else
+        my.parser = FALSE;
+    } 
+    else if (strmatch(option, "nofollow")) {
+      if (value && strlen(value) > 3) {
+        my.nomap->line = (char**)realloc(my.nomap->line, sizeof(char *) * (my.nomap->index + 1));
+        my.nomap->line[my.nomap->index] = (char *)strdup(value);
+        my.nomap->index++;
+      }
+    }
     else if (strmatch(option, "csv")) {
       if (!strncasecmp(value, "true", 4))
         my.csv = TRUE;
@@ -317,12 +425,6 @@ load_conf(char *filename)
     else if (strmatch(option, "logfile")) {
       strncpy(my.logfile, value, sizeof(my.logfile)); 
     } 
-    else if (strmatch(option, "cookies")) {
-      if (strmatch(value, "true"))
-        my.cookies = TRUE;
-      else
-        my.cookies = FALSE;
-    }
     else if (strmatch(option, "concurrent")) {
       if (value != NULL) {
         my.cusers = atoi(value);
@@ -337,12 +439,19 @@ load_conf(char *filename)
         my.reps = 5;
       }
     }
+    else if (strmatch(option, "limit")) {
+      if (value != NULL) {
+        my.limit = atoi(value);
+      } else {
+        my.limit = 255;
+      }
+    }
     else if (strmatch(option, "time")) {
       parse_time(value);
     }
     else if (strmatch(option, "delay")) {
       if (value != NULL) {
-        my.delay = atoi(value);
+        my.delay = atof(value);
       } else {
         my.delay = 1;
       }
@@ -402,7 +511,15 @@ load_conf(char *filename)
       strncpy(my.uagent, value, sizeof(my.uagent));
     }
     else if (strmatch(option, "accept-encoding")) {
-      strncpy(my.encoding, value, sizeof(my.encoding));
+      BOOLEAN compress = FALSE;
+      if ((strstr(value, "gzip") != NULL)||(strstr(value, "compress") != NULL)) {
+        compress = TRUE;
+      }
+      if (compress == TRUE && zlib == FALSE) {
+        NOTIFY(WARNING, "Zip encoding disabled; siege requires zlib support to enable it");
+      } else {
+        strncpy(my.encoding, value, sizeof(my.encoding));
+      }
     }
     #if 1
     else if (!strncasecmp(option, "login", 5)) {
@@ -422,12 +539,6 @@ load_conf(char *filename)
         my.bids = 3;
       }
     }
-    else if (strmatch(option, "username")) {
-      my.username = stralloc(trim(value));
-    }
-    else if (strmatch(option, "password")) {
-      my.password = stralloc(trim(value));
-    }
     else if (strmatch(option, "connection")) {
       if (!strncasecmp(value, "keep-alive", 10))
         my.keepalive = TRUE;
@@ -494,10 +605,10 @@ load_conf(char *filename)
         my.follow = FALSE;
     }
     else if (strmatch(option, "url-escaping")) {
-      if (!strncasecmp(value, "true", 4))
-        my.escape = TRUE;
-      else
+      if (!strncasecmp(value, "false", 5))
         my.escape = FALSE;
+      else
+        my.escape = TRUE;
     } 
     else if (strmatch(option, "zero-data-ok")) {
       if (!strncasecmp(value, "true", 4))
@@ -529,12 +640,11 @@ load_conf(char *filename)
     } else {
       hash_add(H, option, value);
     }
+    xfree(tmp);
     xfree(value);
     xfree(option);
-    free(tmp);
-  } /* end of while line=chomp_line */
-
-  hash_destroy(H);
+  }
+  H = hash_destroy(H);
   fclose(fp);
   return 0;
 }
