@@ -1,8 +1,11 @@
 /**
  * URL Support
  *
- * Copyright (C) 2013-2014 by
+ * Copyright (C) 2013-2015 
  * Jeffrey Fulmer - <jeff@joedog.org>, et al.
+ * Copyright (C) 1999 by
+ * Jeffrey Fulmer - <jeff@joedog.org>.
+ *
  * This file is distributed as part of Siege
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,14 +23,20 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *--
  */
+#ifdef  HAVE_CONFIG_H
+# include <config.h>
+#endif/*HAVE_CONFIG_H*/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <setup.h>
 #include <url.h>
 #include <load.h>
+#include <perl.h>
 #include <date.h>
 #include <util.h>
-#include <joedog/joedog.h>
+#include <memory.h>
+#include <notify.h>
 #include <joedog/boolean.h>
 #include <joedog/defs.h>
 
@@ -52,11 +61,8 @@ struct URL_T
   char *    postdata;
   char *    posttemp;
   char *    conttype;
-  time_t    expires;
-  time_t    modified;
   BOOLEAN   cached;
-  char *    etag;
-  char *    realm;
+  BOOLEAN   redir;
 };
 
 size_t URLSIZE = sizeof(struct URL_T);
@@ -79,16 +85,17 @@ private char *  __url_set_query(URL this, char *str);
 private char *  __url_set_fragment(URL this, char *str);
 private char *  __url_escape(const char *s);
 private METHOD  __url_has_method(const char *url);
-
+private void    __url_replace(char *url, const char *needle, const char *replacement);
 
 URL
 new_url(char *str)
 {
   URL this;
-
   this = xcalloc(sizeof(struct URL_T), 1);
   this->ID = 0;
   this->hasparams = FALSE;
+  this->params    = NULL;
+  this->redir     = FALSE;
   __url_parse(this, str); 
   return this;
 }
@@ -97,6 +104,7 @@ URL
 url_destroy(URL this)
 {
   if (this!=NULL) {
+    xfree(this->url);
     xfree(this->username);
     xfree(this->password);
     xfree(this->hostname);
@@ -107,7 +115,10 @@ url_destroy(URL this)
     xfree(this->request);
     xfree(this->conttype);
     xfree(this->postdata);
-    xfree(this->etag);
+    xfree(this->posttemp);
+    if (this->hasparams==TRUE) {
+      xfree(this->params);
+    }
     xfree(this);
   }
   return NULL;  
@@ -116,7 +127,6 @@ url_destroy(URL this)
 /**
  * URL setters 
  */
-
 void 
 url_set_ID(URL this, int ID)
 {
@@ -124,6 +134,44 @@ url_set_ID(URL this, int ID)
   return;
 }
 
+/**
+ * This function is largely for RE-setting the scheme
+ */
+void
+url_set_scheme(URL this, SCHEME scheme)
+{
+  char *tmp;
+  char *str;
+  int   n;
+  int   len;
+
+  this->scheme = scheme;
+  str = strdup(url_get_scheme_name(this));
+
+  if (this->url != NULL) {
+    tmp = xstrdup(this->url);
+    if (!strncasecmp(tmp, "http:", 5)){
+      n = 7;
+    }
+    if (!strncasecmp(tmp, "https:", 6)){
+      n = 8;
+    }
+    if (!strncasecmp(tmp, "ftp:", 4)){
+      n = 6;
+    }
+    len = strlen(tmp);
+    memmove(tmp, tmp+n, len - n + 1);
+    xfree(this->url);
+    len = strlen(tmp)+strlen(str)+4;
+    this->url = xmalloc(len);
+    memset(this->url, '\0', len);
+    snprintf(this->url, len, "%s://%s", str, tmp);
+    xfree(tmp);
+    xfree(str);
+  }
+  return;
+}
+
 /**
  * if we don't have a hostname at 
  * construction, we can use this 
@@ -139,30 +187,15 @@ url_set_hostname(URL this, char *hostname)
   xfree(this->hostname);
   len = strlen(hostname)+1;
   this->hostname = xmalloc(len);
-  memset(this->hostname, 0, sizeof this->hostname);
+  memset(this->hostname, '\0', len);
   strncpy(this->hostname, hostname, len);
   return;
 }
 
-void
-url_set_last_modified(URL this, char *date) 
+void 
+url_set_redirect(URL this, BOOLEAN redir)
 {
-  this->modified = strtotime(date);
-  return;
-}
-
-void
-url_set_etag(URL this, char *etag)
-{
-  size_t len;
-
-  if (empty(etag)) return;
-
-  len = strlen(etag)+1;
-  this->etag = xmalloc(len);
-  memset(this->etag, 0, sizeof this->etag);
-  strncpy(this->etag, etag, len);
-  return;
+  this->redir = redir;
 }
 
 void 
@@ -176,24 +209,23 @@ url_set_method(URL this, METHOD method) {
   this->method = method;
 }
 
+/**
+ * invoked when post data is read from a file.
+ * see load.c 
+ */
 void
 url_set_postdata(URL this, char *postdata, size_t postlen)
 {
-  this->postlen = postlen;
-
-  if (strlen(postdata) > 0) {
-    this->postdata = malloc(this->postlen);
-    memcpy(this->postdata, postdata, this->postlen);
-    this->postdata[this->postlen] = 0;
-  }
+  this->postlen   = postlen;
+  this->postdata = xmalloc(this->postlen+1);
+  memcpy(this->postdata, postdata, this->postlen);
+  this->postdata[this->postlen] = '\0';
   return;
 }
 
-
 /**
  * URL getters
  */
-
 public int
 url_get_ID(URL this) 
 {
@@ -203,7 +235,7 @@ url_get_ID(URL this)
 public char *
 url_get_absolute(URL this)
 {
-  return this->url;
+  return (this == NULL) ? "NULL" : this->url;
 }
 
 public SCHEME
@@ -320,6 +352,14 @@ url_get_posttemp(URL this) {
 
 public char *
 url_get_conttype(URL this) {
+
+  if (this->conttype == NULL) {
+    if (! empty(my.conttype)) {
+      this->conttype = xstrdup(my.conttype);
+    } else {
+      this->conttype = xstrdup("application/x-www-form-urlencoded");
+    }
+  }
   return this->conttype;
 }
 
@@ -333,6 +373,8 @@ url_get_method_name(URL this) {
   switch (this->method){
     case POST:
       return "POST";
+    case PATCH:
+      return "PATCH";
     case PUT:
       return "PUT";
     case DELETE:
@@ -346,42 +388,10 @@ url_get_method_name(URL this) {
   return "GET";
 }
 
-char *
-url_get_if_modified_since(URL this)
+BOOLEAN  
+url_is_redirect(URL this)
 {
-  if (this->cached == FALSE){
-    return NULL;
-  }
-
-  return timetostr(&this->modified);
-}
-
-char *
-url_get_etag(URL this)
-{
-  char   *tag;
-  size_t len;
-
-  if (empty(this->etag)) return NULL;
-
-  len = strlen(this->etag) + 18;
-  tag = xmalloc(len);
-  memset(tag, 0, sizeof tag);
-
-  snprintf(tag, len, "If-None-Match: %s\015\012", this->etag);
-  return tag;
-}
-
-public char *
-url_get_realm(URL this)
-{
-  return (this->realm!=NULL)?this->realm:"";
-}
-
-public void
-url_set_realm(URL this, char *realm)
-{
-  this->realm = xstrdup(realm);
+  return this->redir;
 }
 
 void
@@ -424,14 +434,9 @@ url_dump(URL this)
     printf("Params:   %s\n", url_get_parameters(this));
   printf("Query:     %s\n", url_get_query(this));
   printf("Fragment:  %s\n", url_get_fragment(this));
-  printf("Post Len:  %d\n", url_get_postlen(this));
+  printf("Post Len:  %d\n", (int)url_get_postlen(this));
   printf("Post Data: %s\n", url_get_postdata(this));
   printf("Cont Type: %s\n", url_get_conttype(this));
-  //time_t    expires;
-  //time_t    modified;
-  //BOOLEAN   cached;
-  //char *    etag;
-  //char *    realm;
   return;
 }
 
@@ -440,28 +445,49 @@ url_normalize(URL req, char *location)
 {
   URL    ret;
   char * url;
-  size_t len = strlen(url_get_absolute(req)) + strlen(location) + 32;
-  if (strchr(location, ':') != NULL) {
+  size_t len;
+
+  /**
+   * Should we just do this for all URLs
+   * or just the ones we parse??
+   */
+  __url_replace(location, "&amp;",  "&");
+  __url_replace(location, "&#038;", "&");
+
+  len = strlen(url_get_absolute(req)) + strlen(location) + 32;
+
+  if (stristr(location, "data:image/gif")) {
+    // stupid CSS tricks
+    return NULL;
+  }
+
+  if (stristr(location, "://")) {
     // it's very likely normalized
     ret = new_url(location);
+
     // but we better test it...
-    if (strchr(url_get_hostname(ret), '.') != NULL) {
+    if (strlen(url_get_hostname(ret)) > 1) {
       return ret;
     }
   }
-  if (strchr(location, '.') != NULL) {
-    // it's *maybe* host/path
+
+  if ((location[0] != '/') && location[0] != '.' && (strchr(location, '.') != NULL && strchr(location, '/') != NULL)) {
+    /**
+     * This is probably host/path; it doesn't start with relevent path 
+     * indicators and it contains the hallmarks of host/path namely at
+     * least one dot and slash
+     */
     ret = new_url(location);
+    url_set_scheme(ret, url_get_scheme(req));
     // so we better test it...
     if (strchr(url_get_hostname(ret), '.') != NULL) {
       return ret;
     }
-    // XXX: do I really need to test for localhost?
   }
 
-  // XXX: 8/20/2014 - YES. Yes, I do.
   if (strstr(location, "localhost") != NULL) {
     ret = new_url(location);
+    url_set_scheme(ret, url_get_scheme(req));
     if (strlen(url_get_hostname(ret)) == 9) {
       // we found and correctly parsed localhost
       return ret;
@@ -482,13 +508,49 @@ url_normalize(URL req, char *location)
       snprintf(url, len, "%s://%s:%d%s", url_get_scheme_name(req), url_get_hostname(req), url_get_port(req), location);
     }
   } else {
-    snprintf(url, len, "%s://%s:%d/%s", url_get_scheme_name(req), url_get_hostname(req), url_get_port(req), location);
+    if (endswith("/", url_get_path(req)) == TRUE) {
+      char *tmp;
+      /**
+       * We're dealing with a req that ends in / and a relative
+       * URL that starts with ./ We want to increment two places
+       * to avoid this path:  /haha/./mama.jpg
+       */
+      if (location[0] == '.' && strlen(location) > 1) {
+        tmp = location+2;
+      } else {
+        tmp = location;
+      }
+      snprintf (  // if the path ends with / we won't need one in the format
+        url, len, 
+       "%s://%s:%d%s%s", 
+       url_get_scheme_name(req), url_get_hostname(req), url_get_port(req), url_get_path(req), tmp
+      );
+    } else {
+      snprintf (  // need to add a slash to separate base path from parsed path/file
+        url, len, 
+        "%s://%s:%d%s/%s", 
+        url_get_scheme_name(req), url_get_hostname(req), url_get_port(req), url_get_path(req), location
+      );
+    }
   }
   ret = new_url(url);
+  url_set_scheme(ret, url_get_scheme(req));
   free(url);
   return ret;
 }
 
+char * 
+url_normalize_string(URL req, char *location) 
+{
+  char *t;
+  URL   u;
+
+  u = url_normalize(req, location);
+  t = strdup(url_get_absolute(u));
+  u = url_destroy(u);
+  return t;
+}
+
 private void
 __url_parse(URL this, char *url)
 {
@@ -507,20 +569,32 @@ __url_parse(URL this, char *url)
   } else {
     ptr = __url_set_absolute(this, url);
   }
-  ptr = __url_set_scheme(this, ptr);
   
+  ptr = __url_set_scheme(this, ptr);
+
   post = strstr(this->url, " POST");
+
   if (! post) {
     post = strstr(this->url, " PUT");
   }
+	
+  if (! post) {
+    post = strstr(this->url, " PATCH");
+  }
 
   if (post != NULL){
     if (!strncasecmp(post," PUT", 4)) {
       this->method = PUT;
+      *post = '\0';
       post += 4;
-    } else {
+    } else if (!strncasecmp(post," POST", 5)) {
       this->method = POST;
+      *post = '\0';
       post += 5;
+    } else {
+      this->method = PATCH;
+      *post = '\0';
+      post += 6;
     }
     __parse_post_data(this, post);
   } else {
@@ -554,9 +628,8 @@ __parse_post_data(URL this, char *datap)
   if (*datap == '<') {
     datap++;
     load_file(this, datap);
-    this->file = xmalloc(strlen(datap)+1);
-    memset(this->file, '\0', strlen(datap)+1);
-    memcpy(this->file, datap, strlen(datap));
+    datap = __url_set_path(this, datap);
+    datap = __url_set_file(this, datap);
     return;
   } else {
     this->postdata = xstrdup(datap);
@@ -578,25 +651,46 @@ __parse_post_data(URL this, char *datap)
 private char *
 __url_set_absolute(URL this, char *url)
 {
+  int    n;
   size_t len;
   char   *slash;
-  //char *ptr = url;
+  char   scheme[16];
 
   if (empty(url)) return NULL;
 
+  memset(scheme, '\0', 16);
+
+  if (!strncasecmp(url, "http:", 5)){
+    n = 7;
+    strncpy(scheme, "http", 4);
+  }
+  if (!strncasecmp(url, "https:", 6)){
+    n = 8;
+    strncpy(scheme, "https", 5);
+  }
+  if (!strncasecmp(url, "ftp:", 4)){
+    n = 6;
+    strncpy(scheme, "ftp", 3);
+  }
+  if (strlen(scheme) < 3) {
+    // A scheme wasn't supplied; we'll use http by default.
+    n = 7;
+    strncpy(scheme, "http", 4);
+  }
+
   len = strlen(url)+5;
   if (!__url_has_scheme(url)) {
-    this->url = xmalloc(len+7);
-    memset(this->url, '\0', sizeof this->url);
+    this->url = xmalloc(len+n);
+    memset(this->url, '\0', len+n);
     slash = strstr(url, "/");
     if (slash) {
-      snprintf(this->url, len+7, "http://%s", url);
+      snprintf(this->url, len+n, "%s://%s", scheme, url);
     } else {
-      snprintf(this->url, len+7, "http://%s/", url);
+      snprintf(this->url, len+n, "%s://%s/", scheme, url);
     }
   } else {
     this->url = xmalloc(len);
-    memset(this->url, '\0', sizeof this->url);
+    memset(this->url, '\0', len);
     snprintf(this->url, len, "%s", url);
   }
   return this->url;
@@ -744,17 +838,24 @@ private char *
 __url_set_hostname(URL this, char *str)
 {
   int i;
+  int n;
+  int len;
+
+  if (startswith("//", str)) {
+    n   = 2;
+    len = strlen(str);
+    memmove(str, str+n, len - n + 1);
+  }
 
   /* skip to end, slash, or port colon */
-  for(i = 0; str[i] && str[i] != '/' && str[i] != ':'; i++);
+  for (i = 0; str[i] && str[i] != '/' && str[i] != '#' && str[i] != ':'; i++);
 
   this->hostname = xmalloc(i + 1);
+  memset(this->hostname, '\0', i+1);
   memcpy(this->hostname, str, i);
 
-  this->hostname[i] = '\0';
-
   /* if there's a port */
-  if(str[i] == ':'){
+  if (str[i] == ':') {
     str += i + 1;
   } else {
     str += i;
@@ -797,14 +898,31 @@ __url_set_port(URL this, char *str)
 private char *
 __url_set_path(URL this, char *str)
 {
-  int  i;    // capture the lenght of the path
-  int  j;    // capture the length of the request (sans frag)
+  int   i;    // capture the lenght of the path
+  int   j;    // capture the length of the request (sans frag)
+  char *c;
 
-  for(i = strlen(str); i > 0 && str[i] != '/'; i--);
-  for(j = 0; str[j] && (str[j] != '#' && !isspace(str[j])); j++);
+  if (str != NULL && str[0] == '#') {
+    // WTF'ery. We probably have this: www.joedog.org#haha
+    this->request = xstrdup("/");
+    return str; 
+  }
 
+  this->request = xstrdup(str);
 
-  if(str[i] != '/'){
+  /**
+   * Does the request have a fragment? 
+   * Let's whack that annoyance off...
+   */
+  c = (char *)strstr(this->request, "#");
+  if (c) {
+   *c = '\0';
+  }
+
+  for (i = strlen(str); i > 0 && str[i] != '/'; i--);
+  for (j = 0; str[j] && (str[j] != '#' && !isspace(str[j])); j++);
+
+  if (str[i] != '/') {
     this->path    = xmalloc(2);
     this->request = xmalloc(2);
     strncpy(this->path,    "/", 2);
@@ -813,16 +931,9 @@ __url_set_path(URL this, char *str)
     this->request[1] = '\0';
   } else {
     this->path    = xmalloc(i+2);
-    this->request = xmalloc(j+2);
     memcpy(this->path, str, i+1);
-    memcpy(this->request, str, j+1);
     this->path[i] = '/';
     this->path[i + 1]    = '\0';
-    if (this->request[j]=='#') {
-      this->request[j]   = '\0'; // lop the #
-    } else {
-      this->request[j+1] = '\0'; // no frag
-    }
   }
   trim(this->request);
   str += i + 1;
@@ -1035,7 +1146,7 @@ __url_has_method(const char *url)
    unsigned int i = 0;
    const char * r = NULL;
    static const char* const methods[] = {
-     " GET", " HEAD", " POST", " PUT", " TRACE", " DELETE", " OPTIONS", " CONNECT"
+     " GET", " HEAD", " POST", " PUT", " TRACE", " DELETE", " OPTIONS", " CONNECT", " PATCH"
    };
 
    for (i = 0; i < sizeof(methods) / sizeof(methods[0]); i++) {
@@ -1113,3 +1224,41 @@ __url_escape(const char *s)
   return newstr;
 }
 
+
+private void
+__url_replace(char *url, const char *needle, const char *replacement)
+{
+  char   buf[4096] = {0};
+  char  *ins       = &buf[0];
+  char  *str       = NULL;
+  const char *tmp  = url;
+  size_t nlen = strlen(needle);
+  size_t rlen = strlen(replacement);
+
+  while (1) {
+    const char *p = strstr(tmp, needle);
+
+    if (p == NULL) {
+      strcpy(ins, tmp);
+      break;
+    }
+
+    memcpy(ins, tmp, p - tmp);
+    ins += p - tmp;
+
+    memcpy(ins, replacement, rlen);
+    ins += rlen;
+    tmp = p + nlen;
+  }
+  if (strlen(buf) > strlen(url)){
+    str = (char *)realloc(url, strlen(buf)+1);
+    if (str == NULL) {
+      return;
+    }
+    url = str;
+    memset(url, '\0', strlen(buf)+1);
+  } else {
+    memset(url, '\0', strlen(url));
+  }
+  strncpy(url, buf, strlen(buf));
+}
